#!/usr/bin/python3

import dbus
import logging
import os
from datetime import datetime
import json
import hashlib
from binascii import a2b_hex
import base64
import subprocess
from Crypto.PublicKey import RSA
from Crypto.Cipher import PKCS1_OAEP
from .RMOsInfo import RMOsInfo
from .RMConf import RMConf
from .RMCom import RMCom


DBUS_INTERFACE = 'com.kylin.daq.interface'
DBUS_PATH = '/com/kylin/daq'
DBUS_SERVICE = 'com.kylin.daq'

class UniqueKey():
    keyvalue = "LS0tLS1CRUdJTiBQVUJMSUMgS0VZLS0tLS0KTUlJQklqQU5CZ2txaGtpRzl3MEJBUUVGQUFPQ0FR\
OEFNSUlCQ2dLQ0FRRUFzdW1NTFJEdlFNb0tEQkRJODRqSgpqc1A0Mk55V0pWVEZob2Jra3ZiT05j\
dExYTXVzRmo2TzJUblZYU3Z6VlVLSjRqZkpwT2l2WEphOVB5Z2wzYTRnClBzSU40enNCMEdOY0tr\
R3VsS2RrV2x6S3lWQ2xlTzhiQnN6SjkwbTc3cWF6YWg3a1A0TUl0WTVFczBpSkpiR0oKN1MxcERj\
MlJkNnVFQWJLaXJyRTFlNzlFTEd4am5VN2V5NWkyRDE2WWJoZEQwZ2lNa2RHR3piQXBKTWZWRVJR\
TQo1NXorMFVqdS8zSFJhNFY3b3p2TGRPRE5HUURaeWNJU0l3VHBLbFR3RjBxazdCNjVhTUlJenQ1\
dnhOK1lxYU1GClppZFRLNzcxNjdqNEExZ3F3MG45bjlybWVXUGRWZ3dudnRtVXp4Q1krNk05SXpK\
TDI3eWpRUTV1WGQ3RVdMT3IKbndJREFRQUIKLS0tLS1FTkQgUFVCTElDIEtFWS0tLS0tCg=="

class RMDaq(RMCom):

    def __init__(self):
        self.baseInfoDict={}
        self.tid = ""
        self.osInfo = RMOsInfo()
        self.baseInfoDict["uuid"] = ''
        self.baseInfoDict["newSysVersion"] = ''
        self.baseInfoDict["oldSysVersion"] = ''
        self.baseInfoDict["step"] = ''
        self.baseInfoDict["stepReason"] = ''

    def _dict_to_json(self,dict):
        #字典转换为json格式字符串
        jsonStr = ''
        try:
            jsonStr = json.dumps(dict)
        except Exception as e:
            logging.info("[RMDAQ]: dict to json err[%s]"%(e))
        return jsonStr

    def _sha256_Value(self, jsonStr):
        # 计算sha256值
        hsobj = hashlib.sha256()
        try:
            hsobj.update(jsonStr.encode("utf-8"))
        except ValueError as e:
            logging.info("[RMDAQ]: SHA256 value error[%s]."%(e))
        except Exception as e:
            logging.info("[RMDAQ]: cal sha256 err[%s]"%(e))
        
        return hsobj.hexdigest()

    def _encode_base64(self, value):
        # 将value进行RSA加密并base64转码
        key = UniqueKey()
        try:
            # 计算hex值
            value_hex = a2b_hex(value)
            # 加载公钥,填充格式OAEP
            uniqueKey = key.keyvalue.encode('utf-8')
            uniqueKeyorig = base64.b64decode(uniqueKey)  # 公钥文件
            rsa_pubkey = RSA.importKey(uniqueKeyorig)    # RSA公钥
            oaep_pub = PKCS1_OAEP.new(rsa_pubkey)        # OAEP填充
            # 加密数据
            encodemsg = oaep_pub.encrypt(value_hex)
            # 加密数据Base64转码
            enMsg = base64.b64encode(encodemsg)   
        except ValueError:
            logging.info("[RMDAQ]: Value error: %s."%(value))
        except TypeError:
            logging.info("[RMDAQ]: RSA key has no private half.")
        return enMsg

    def send_json(self, func, UploadMessage):
        ret = 0
        PackageInfo = {}
        PackageInfo["packageName"] = "revisions-manager"
        PackageInfo["messageType"] = "collectInfo"
        PackageInfo["tid"] = self.tid
        json_PackageInfo = self._dict_to_json(PackageInfo)

        UploadMessage["createTimeStamp"] = datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f")[:-3]
        json_UploadMessage = self._dict_to_json(UploadMessage)
        shaValue = self._sha256_Value(json_UploadMessage) 
        encodeMsg = self._encode_base64(shaValue)

        for i in range(1,5):

            try:
                ret, retTid = func(json_PackageInfo, json_UploadMessage, encodeMsg)
            except AttributeError:
                logging.info("[RMDAQ]: Call UploadMessage: Attribute Error.")
            
            if ret == 2:
                self.tid = retTid

            if ret != 0 and ret != 2:
                logging.info("[RMDAQ]: Times[%d] send Json err[%d]."%(i, ret))
            else:
                logging.info("[RMDAQ]: Times[%d] send Json successfully[%d]."%(i, ret))
                break

        logging.info("[RMDAQ]: send ret[%d].", ret)
        return ret

    
    def send_all_info(self):
        self.get_all()
        try:
            bus = dbus.SystemBus()
            proxy = bus.get_object(DBUS_SERVICE, DBUS_PATH)
            bus_interface = dbus.Interface(proxy, dbus_interface=DBUS_INTERFACE)

            func = bus_interface.get_dbus_method("UploadMessage")
            if func == None:
                logging.info("[RMDAQ]: Func is none.")

            self.send_json(func, self.baseInfoDict)
        except Exception as e:
            logging.info("[RMDAQ]: Get bus err[%s]."%(e))

    def get_current_version(self):
        self.baseInfoDict["newSysVersion"] = self.osInfo.get_pretty_name() + "-" + self.osInfo.get_release_id()

    def get_old_version(self):
        filePath = "/fs.old/etc/os-release"
        if not os.path.exists(filePath):
            logging.error("[RMDAQ]: [%s] not exists.", filePath)
            return
        oldosInfo = RMOsInfo(filePath)
        releaseId = oldosInfo.get_release_id()
        if len(releaseId) == 0:
            self.baseInfoDict["oldSysVersion"] = oldosInfo.get_pretty_name()
        else:
            self.baseInfoDict["oldSysVersion"] = oldosInfo.get_pretty_name() + "-" + oldosInfo.get_release_id()

    def get_tools_step(self):
        if not os.path.exists(self.RM_STATE_PATH):
            logging.error("[RMDAQ]: [%s] not exists.", self.RM_STATE_PATH)
            return
        stateConf = RMConf(self.RM_STATE_PATH)
        self.baseInfoDict["step"] = stateConf.get_key(self.SEC_STATE, self.KEY_STEP)
        self.baseInfoDict["stepReason"] = stateConf.get_key(self.SEC_STATE, self.KEY_REASEON)

    def get_uuid(self):
        cmd = "/usr/sbin/dmidecode -s system-uuid | grep '[0-9a-fA-F]\\{8\\}-[0-9a-fA-F]\\{4\\}-[0-9a-fA-F]\\{4\\}-[0-9a-fA-F]\\{4\\}-[0-9a-fA-F]\\{12\\}'"
        cmdRet = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        if cmdRet.returncode != 0:
            logging.error("[RMDAQ]: cmd[%s] ret[%d] failed as [%s].", cmd, cmdRet.returncode, cmdRet.stdout.decode())
            return
        
        self.baseInfoDict["uuid"] = cmdRet.stdout.decode()
            


    def get_all(self):
        try:
            self.get_uuid()
            self.get_current_version()
            self.get_old_version()
            self.get_tools_step()
        except Exception as e:
            logging.error("[RMDAQ]: get info failed as [%s].", e)

    def set_step_and_reason(self, step, reason):
        try:
            self.baseInfoDict["step"] = step
            self.baseInfoDict["stepReason"] = reason
            stateConf = RMConf(self.RM_STATE_PATH)
            stateConf.change_key_value(self.SEC_STATE, self.KEY_STEP, step)
            stateConf.change_key_value(self.SEC_STATE, self.KEY_REASEON, reason)
        except Exception as e:
            logging.error("[RMDAQ]: Set_step_and_reason err[%s].", e)



if __name__ == "__main__":
    info = RMDaq()

    info.send_all_info()



    


